From: Andrew Morton <akpm@linux-foundation.org>
To: mm-commits@vger.kernel.org, willy@infradead.org, vbabka@suse.cz,
sudeep.holla@arm.com, pasha.tatashin@soleen.com, mhocko@suse.com,
mgorman@techsingularity.net, jweiner@redhat.com,
david@redhat.com, dave.hansen@linux.intel.com, cl@linux.com,
arjan@linux.intel.com, ying.huang@intel.com,
akpm@linux-foundation.org
Subject: [merged mm-stable] mm-pcp-avoid-to-reduce-pcp-high-unnecessarily.patch removed from -mm tree
Date: Fri, 06 Oct 2023 14:48:55 -0700 [thread overview]
Message-ID: <20231006214857.AA06FC433C8@smtp.kernel.org> (raw)
The quilt patch titled
Subject: mm, pcp: avoid to reduce PCP high unnecessarily
has been removed from the -mm tree. Its filename was
mm-pcp-avoid-to-reduce-pcp-high-unnecessarily.patch
This patch was dropped because it was merged into the mm-stable branch
of git://git.kernel.org/pub/scm/linux/kernel/git/akpm/mm
------------------------------------------------------
From: Huang Ying <ying.huang@intel.com>
Subject: mm, pcp: avoid to reduce PCP high unnecessarily
Date: Tue, 26 Sep 2023 14:09:10 +0800
In PCP high auto-tuning algorithm, to minimize idle pages in PCP, in
periodic vmstat updating kworker (via refresh_cpu_vm_stats()), we will
decrease PCP high to try to free possible idle PCP pages. One issue is
that even if the page allocating/freeing depth is larger than maximal PCP
high, we may reduce PCP high unnecessarily.
To avoid the above issue, in this patch, we will track the minimal PCP
page count. And, the periodic PCP high decrement will not more than the
recent minimal PCP page count. So, only detected idle pages will be
freed.
On a 2-socket Intel server with 224 logical CPU, we run 8 kbuild instances
in parallel (each with `make -j 28`) in 8 cgroup. This simulates the
kbuild server that is used by 0-Day kbuild service. With the patch, The
number of pages allocated from zone (instead of from PCP) decreases 21.4%.
Link: https://lkml.kernel.org/r/20230926060911.266511-10-ying.huang@intel.com
Signed-off-by: "Huang, Ying" <ying.huang@intel.com>
Cc: Mel Gorman <mgorman@techsingularity.net>
Cc: Vlastimil Babka <vbabka@suse.cz>
Cc: David Hildenbrand <david@redhat.com>
Cc: Johannes Weiner <jweiner@redhat.com>
Cc: Dave Hansen <dave.hansen@linux.intel.com>
Cc: Michal Hocko <mhocko@suse.com>
Cc: Pavel Tatashin <pasha.tatashin@soleen.com>
Cc: Matthew Wilcox <willy@infradead.org>
Cc: Christoph Lameter <cl@linux.com>
Cc: Arjan van de Ven <arjan@linux.intel.com>
Cc: Sudeep Holla <sudeep.holla@arm.com>
Signed-off-by: Andrew Morton <akpm@linux-foundation.org>
---
include/linux/mmzone.h | 1 +
mm/page_alloc.c | 15 ++++++++++-----
2 files changed, 11 insertions(+), 5 deletions(-)
--- a/include/linux/mmzone.h~mm-pcp-avoid-to-reduce-pcp-high-unnecessarily
+++ a/include/linux/mmzone.h
@@ -680,6 +680,7 @@ enum zone_watermarks {
struct per_cpu_pages {
spinlock_t lock; /* Protects lists field */
int count; /* number of pages in the list */
+ int count_min; /* minimal number of pages in the list recently */
int high; /* high watermark, emptying needed */
int high_min; /* min high watermark */
int high_max; /* max high watermark */
--- a/mm/page_alloc.c~mm-pcp-avoid-to-reduce-pcp-high-unnecessarily
+++ a/mm/page_alloc.c
@@ -2166,19 +2166,20 @@ static int rmqueue_bulk(struct zone *zon
*/
int decay_pcp_high(struct zone *zone, struct per_cpu_pages *pcp)
{
- int high_min, to_drain, batch;
+ int high_min, decrease, to_drain, batch;
int todo = 0;
high_min = READ_ONCE(pcp->high_min);
batch = READ_ONCE(pcp->batch);
/*
* Decrease pcp->high periodically to try to free possible
* idle PCP pages. And, avoid to free too many pages to
* control latency.
*/
if (pcp->high > high_min) {
+ decrease = min(pcp->count_min, pcp->high / 5);
pcp->high = max3(pcp->count - (batch << PCP_BATCH_SCALE_MAX),
- pcp->high * 4 / 5, high_min);
+ pcp->high - decrease, high_min);
if (pcp->high > high_min)
todo++;
}
@@ -2191,6 +2192,8 @@ int decay_pcp_high(struct zone *zone, st
todo++;
}
+ pcp->count_min = pcp->count;
+
return todo;
}
@@ -2828,6 +2831,8 @@ struct page *__rmqueue_pcplist(struct zo
page = list_first_entry(list, struct page, pcp_list);
list_del(&page->pcp_list);
pcp->count -= 1 << order;
+ if (pcp->count < pcp->count_min)
+ pcp->count_min = pcp->count;
} while (check_new_pages(page, order));
return page;
_
Patches currently in -mm which might be from ying.huang@intel.com are
mm-fix-draining-remote-pageset.patch
reply other threads:[~2023-10-06 21:49 UTC|newest]
Thread overview: [no followups] expand[flat|nested] mbox.gz Atom feed
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20231006214857.AA06FC433C8@smtp.kernel.org \
--to=akpm@linux-foundation.org \
--cc=arjan@linux.intel.com \
--cc=cl@linux.com \
--cc=dave.hansen@linux.intel.com \
--cc=david@redhat.com \
--cc=jweiner@redhat.com \
--cc=linux-kernel@vger.kernel.org \
--cc=mgorman@techsingularity.net \
--cc=mhocko@suse.com \
--cc=mm-commits@vger.kernel.org \
--cc=pasha.tatashin@soleen.com \
--cc=sudeep.holla@arm.com \
--cc=vbabka@suse.cz \
--cc=willy@infradead.org \
--cc=ying.huang@intel.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -2166,19 +2166,20 @@ static int rmqueue_bulk(struct zone *zon
batch = READ_ONCE(pcp->batch);
/*
- * Decrease pcp->high periodically to try to free possible
- * idle PCP pages. And, avoid to free too many pages to
- * control latency.
+ * Decrease pcp->high periodically to free idle PCP pages counted
+ * via pcp->count_min. And, avoid to free too many pages to
+ * control latency. This caps pcp->high decrement too.
*/
if (pcp->high > high_min) {
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.